/*
Copyright (C) 2014 Paul Boddie

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
more details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
*/

module cartridge()
{
	$fn = 50;

	/*
	Rounding/fillet radius and additional margin of subtracted
	material. The additional margin helps avoid geometry problems.
	*/

	rr = 1;
	ro = rr;
	extra = 0.1;

	module fillet(r, h) {
		translate([0, 0, -h/2])
			difference() {
				cube([r + extra, r + extra, h + extra]);
				cylinder(r = r, h = h);
			}
	}

	/*
	Justify an object of the given dimensions, according to the given
	factors (where 1 indicates moving the object to the positive side of an
	axis, and -1 indicates moving it to the negative side of an axis).

	NOTE: child should eventually be replaced by children.
	*/
	module justify(width, depth, height, wdir, ddir, hdir) {
		translate([
			wdir * width / 2,
			ddir * depth / 2,
			hdir * height / 2])
			child();
	}

	/*
	Make a cuboid of the given dimensions, justifying it according to the given
	factors, and moving it to the specified coordinates.

	NOTE: Usage of justify within this module will not work due to recursion
	NOTE: limitations in openscad, potentially removed in more recent
	NOTE: releases. Thus, the justify transform is merged in here.
	*/
	module cube_at(width, depth, height, wdir, ddir, hdir, x, y, z) {
		translate([
			x + wdir * width / 2,
			y + ddir * depth / 2,
			z + hdir * height / 2])
			cube([width, depth, height], center = true);
	}

	/* Cartridge dimensions. */

	width = 89.0;
	height = 68.5;
	depth = 16.5;
	front_depth = 6.5;
	back_depth = 11.0;

	/* Label details. */

	front_label_width = 83.0;
	front_label_height = 46.0;
	front_label_depth = 1.0;
	front_label_offset_from_bottom = 19.5;

	top_label_width = front_label_width;
	top_label_height = 11.5; /* the height as seen from above */
	top_label_depth = front_label_depth;
	top_label_offset_from_front = 2.5;

	/* The groove around the sides and top. */

	groove_width = 2.5; /* how much the groove cuts out of the back */
	groove_depth = 1.0; /* how deep the groove goes into each side */

	/* Additional cutting to mate the back and front. */

	top_groove_width = 1.0;
	top_groove_depth = 2.0;

	/* Space for the back inside the front. */

	inner_top_front_cutout_width = 87.0;
	inner_top_front_cutout_depth = 1.0;
	inner_top_front_cutout_offset = 2.0;

	back_cavity_width = 68.0;
	back_cavity_inner_width = 65.0;
	back_cavity_offset_from_left = 10.5;
	back_cavity_inner_offset_from_left = 12.0;
	back_cavity_height = 13.5;
	back_cavity_inner_height = 12.0;
	back_cavity_depth = 1.5;

	inner_back_cavity_offset = 1.0;
	inner_back_cavity_offset_from_left = 2.0;
	inner_back_slope_offset_from_left = 10.0;
	inner_back_slope_width = 2.5;
	inner_back_slope_depth = 2.5;

	inner_back_edge_offset = 2.0;
	inner_back_edge_width = 69.0;
	inner_back_edge_height = 3.0;
	inner_back_edge_depth = 1.5;

	inner_front_edge_offset = 0.5;
	inner_front_edge_width = 87.0;
	inner_front_edge_height = 3.0;
	inner_front_edge_depth = 1.5;

	bottom_from_base = 13.5;

	edge_connector_cutout_front_offset = 1.0;
	edge_connector_cutout_front_depth = 1.0;
	edge_connector_cutout_front_width = 15.0;
	edge_connector_cutout_back_depth = 3.0;
	edge_connector_cutout_back_width = 57.0;

	/*
	Edge connectors are themselves 0.05" or approximately 1.27mm in
	thickness according to the Acorn Electron Cartridge Interface Specification
	(Acorn Support Application Group Note 014).
	*/

	/* Side thicknesses. */

	front = 2;
	back = 3.5;
	top = 3;
	front_left = 1;
	front_right = 1;
	back_left = 2;
	back_right = 2;
	bottom = 2;

	/* Extra internal features. */

	pcb_back_support_width = 1.0;
	pcb_back_support_depth = back_depth -
		edge_connector_cutout_back_depth;
	pcb_back_support_height = height - bottom_from_base - top - bottom;

	pcb_front_support_width = 1.0;
	pcb_front_support_depth = front_depth;
	pcb_front_support_height = pcb_back_support_height;

	/*
	Features measured from the Stardot Dual ROM Adaptor	cartridge board
	dimensions diagram.
	*/

	pcb_back_support_bump_width = pcb_front_support_width;
	pcb_back_support_bump_depth = 1.5;
	pcb_back_support_left_bump_height = 14.3;
	pcb_back_support_right_bump_height = 11.7;
	pcb_back_support_left_bump_offset_from_bottom = 26.0 - 11.8;
	pcb_back_support_right_bump_offset_from_bottom = 28.6 - 11.8;

	translate([-width * 0.6, 0, 0])
	difference() {

		/* The cartridge surfaces. */

		union() {

			/* Front portion. */

			translate([0, -front_depth + front / 2, 0])
				cube([width, front, height], center = true);
			translate([-width / 2 + front_left / 2, -front_depth / 2, 0])
				cube([front_left, front_depth, height], center = true);
			translate([width / 2 - front_right / 2, -front_depth / 2, 0])
				cube([front_right, front_depth, height], center = true);
			translate([0, -front_depth / 2, height / 2 - top / 2])
				cube([width, front_depth, top], center = true);
			difference() {

				/* Floor of cartridge. */

				cube_at(width, front_depth, bottom,
					0, -1, 1,
					0, 0, -height / 2 + bottom_from_base);

				/* Left cutout. */

				cube_at(edge_connector_cutout_front_width,
					edge_connector_cutout_front_depth,
					bottom,
					1, -1, 1,
					-width / 2 + edge_connector_cutout_front_offset,
					0,
					-height / 2 + bottom_from_base);

				/* Right cutout. */

				cube_at(edge_connector_cutout_front_width,
					edge_connector_cutout_front_depth,
					bottom,
					-1, -1, 1,
					width / 2 - edge_connector_cutout_front_offset,
					0,
					-height / 2 + bottom_from_base);
			}

			/* PCB supports. */

			cube_at(pcb_front_support_width,
					pcb_front_support_depth,
					pcb_front_support_height,
					1, -1, 1,
					-edge_connector_cutout_back_width / 2,
					0,
					-height / 2 + bottom + bottom_from_base);

			cube_at(pcb_front_support_width,
					pcb_front_support_depth,
					pcb_front_support_height,
					-1, -1, 1,
					edge_connector_cutout_back_width / 2,
					0,
					-height / 2 + bottom + bottom_from_base);
		}

		/* Label insets. */

		union() {

			/* Front label. */

			translate([-front_label_width / 2, -front_depth,
				front_label_offset_from_bottom - height / 2])
				cube([front_label_width, front_label_depth,
					front_label_height]);

			/* Top label. */

			translate([-top_label_width / 2,
				-front_depth + top_label_offset_from_front,
				height / 2 - top_label_depth])
				cube([top_label_width, top_label_height,
					top_label_depth]);
		}

		/* Inner front edge cavity. */

		translate([inner_front_edge_width / 2,
			-front_depth + inner_front_edge_offset, -height / 2])
			rotate([0, -90, 0])
				linear_extrude(height = inner_front_edge_width)
					polygon([
						[-extra, 0],
						[0, 0],
						[inner_front_edge_height, inner_front_edge_depth],
						[-extra, inner_front_edge_depth],
						]);

		/* Inner top cutout for the top of the back portion. */

		translate([0, -inner_top_front_cutout_depth / 2, height / 2 -
			inner_top_front_cutout_offset -
			inner_top_front_cutout_depth / 2])
			cube([inner_top_front_cutout_width,
				inner_top_front_cutout_depth,
				inner_top_front_cutout_depth], center = true);

		/* Fillets to round off the edges. */

		union() {

			/* Top left and right rounding. */
		
			translate([-width / 2 + ro, -front_depth / 2, height / 2 - ro])
				rotate([0, 0, 180])
					rotate([90, 0, 0])
						fillet(rr, front_depth);
			translate([width / 2 - ro, -front_depth / 2, height / 2 - ro])
				rotate([90, 0, 0])
					fillet(rr, front_depth);

			/* Top front rounding. */
		
			translate([0, -front_depth + ro, height / 2 - ro])
				rotate([0, 0, 180])
					rotate([0, -90, 0])
						fillet(rr, width);

			/* Edge rounding. */
		
			translate([-width / 2 + ro, -front_depth + ro, 0])
				rotate([0, 0, 180])
					fillet(rr, height);
			translate([width / 2 - ro, -front_depth + ro, 0])
				rotate([0, 0, 270])
					fillet(rr, height);
		}
	}

	translate([width * 0.6, 0, 0])
	rotate([0, 0, 180])
	difference() {

		/* The cartridge surfaces. */

		union() {

			/* Back portion. */

			translate([0, back_depth - back / 2, 0])
				cube([width, back, height], center = true);
			translate([-width / 2 + back_left / 2, back_depth / 2, 0])
				cube([back_left, back_depth, height], center = true);
			translate([width / 2 - back_right / 2, back_depth / 2, 0])
				cube([back_right, back_depth, height], center = true);
			translate([0, back_depth / 2, height / 2 - top / 2])
				cube([width, back_depth, top], center = true);
			difference() {

				/* Floor of cartridge. */

				cube_at(width, back_depth, bottom,
					0, 1, 1,
					0, 0, -height / 2 + bottom_from_base);

				/* Edge connector cutout. */

				cube_at(edge_connector_cutout_back_width,
					edge_connector_cutout_back_depth,
					bottom,
					0, 1, 1,
					0, 0, -height / 2 + bottom_from_base);
			}

			/* PCB supports. */

			translate([-edge_connector_cutout_back_width / 2,
				edge_connector_cutout_back_depth,
				-height / 2 + bottom + bottom_from_base])
				justify(pcb_back_support_width,
					pcb_back_support_depth,
					pcb_back_support_height,
					1, 1, 1)
					union() {
						cube([pcb_back_support_width,
							pcb_back_support_depth,
							pcb_back_support_height], center = true);
						cube_at(pcb_back_support_bump_width,
							pcb_back_support_bump_depth,
							pcb_back_support_left_bump_height,
							0, -1, 1,
							0,
							-pcb_back_support_depth / 2,
							-pcb_back_support_height / 2 +
								pcb_back_support_left_bump_offset_from_bottom);
					}

			translate([edge_connector_cutout_back_width / 2,
				edge_connector_cutout_back_depth,
				-height / 2 + bottom + bottom_from_base])
				justify(pcb_back_support_width,
					pcb_back_support_depth,
					pcb_back_support_height,
					-1, 1, 1)
					union() {
						cube([pcb_back_support_width,
							pcb_back_support_depth,
							pcb_back_support_height], center = true);
						cube_at(pcb_back_support_bump_width,
							pcb_back_support_bump_depth,
							pcb_back_support_right_bump_height,
							0, -1, 1,
							0,
							-pcb_back_support_depth / 2,
							-pcb_back_support_height / 2 +
								pcb_back_support_right_bump_offset_from_bottom);
					}
		}

		/* Label insets. */

		union() {

			/* Top label. */

			translate([-top_label_width / 2,
				-front_depth + top_label_offset_from_front,
				height / 2 - top_label_depth])
				cube([top_label_width, top_label_height,
					top_label_depth]);
		}

		/* Top and side grooves, positioned in the back portion. */

		union() {

			/* Left groove. */

			translate([-width / 2 + groove_depth / 2, groove_width / 2, 0])
				cube([groove_depth, groove_width, height],
					center = true);

			/* Right groove. */

			translate([width / 2 - groove_depth / 2, groove_width / 2, 0])
				cube([groove_depth, groove_width, height],
					center = true);

			/* Top grooves. */

			translate([0, groove_width / 2, height / 2 - groove_depth / 2])
				cube([width, groove_width, groove_depth],
					center = true);

			translate([0, top_groove_width / 2,
				height / 2 - top_groove_depth / 2])
				cube([width, top_groove_width, top_groove_depth],
					center = true);
		}

		/* Back cavity. */

		intersection() {

			/* From the bottom upwards. */

			translate([0, back_depth, -height / 2])
				linear_extrude(height = back_cavity_height)
					translate([-width / 2, 0, 0])
						polygon([
							[back_cavity_offset_from_left, 0],
							[back_cavity_inner_offset_from_left,
								-back_cavity_depth],
							[back_cavity_inner_offset_from_left +
								back_cavity_inner_width,
								-back_cavity_depth],
							[back_cavity_offset_from_left +
								back_cavity_width, 0]
							]);

			/* From left to right. */

			translate([back_cavity_width / 2, back_depth, -height / 2])
				rotate([0, -90, 0])
					linear_extrude(height = back_cavity_width)
						polygon([
							[-extra, -back_cavity_depth],
							[back_cavity_inner_height,
								-back_cavity_depth],
							[back_cavity_height, 0],
							[-extra, 0]
							]);
		}

		/* Inner back cavities. */

		translate([0, back_depth - inner_back_cavity_offset, -height / 2])
			linear_extrude(height = bottom_from_base)
				translate([-width / 2, 0, 0])
					polygon([
						[inner_back_cavity_offset_from_left, 0],
						[inner_back_slope_offset_from_left, 0],
						[inner_back_slope_offset_from_left +
							inner_back_slope_width,
							-inner_back_slope_depth],
						[inner_back_cavity_offset_from_left,
							-inner_back_slope_depth]
						]);

		translate([0, back_depth - inner_back_cavity_offset, -height / 2])
			linear_extrude(height = bottom_from_base)
				translate([-width / 2, 0, 0])
					polygon([
						[width - inner_back_slope_offset_from_left, 0],
						[width - inner_back_cavity_offset_from_left, 0],
						[width - inner_back_cavity_offset_from_left,
							-inner_back_slope_depth],
						[width - inner_back_slope_offset_from_left -
							inner_back_slope_width,
							-inner_back_slope_depth]
						]);

		/* Inner back edge cavity. */

		translate([inner_back_edge_width / 2,
			back_depth - inner_back_edge_offset, -height / 2])
			rotate([0, -90, 0])
				linear_extrude(height = inner_back_edge_width)
					polygon([
						[-extra, -inner_back_edge_depth],
						[inner_back_edge_height, -inner_back_edge_depth],
						[0, 0],
						[-extra, 0]
						]);

		/* Fillets to round off the edges. */

		union() {

			/* Top left and right rounding. */
		
			translate([-width / 2 + ro, back_depth / 2, height / 2 - ro])
				rotate([0, 0, 180])
					rotate([90, 0, 0])
						fillet(rr, back_depth);
			translate([width / 2 - ro, back_depth / 2, height / 2 - ro])
				rotate([90, 0, 0])
					fillet(rr, back_depth);
		
			/* Top back rounding. */
		
			translate([0, back_depth - ro, height / 2 - ro])
				rotate([0, -90, 0])
					fillet(rr, width);
		
			/* Edge rounding. */
		
			translate([width / 2 - ro, back_depth - ro, 0])
				fillet(rr, height);
			translate([-width / 2 + ro, back_depth - ro, 0])
				rotate([0, 0, 90])
					fillet(rr, height);
		}
	}
}

cartridge();

